// Mathlib.cpp file .cpp

// include prototypes
#include "Arduino.h"

// include component
#include "MatrixLib.h"
//constructor
MatrixObj::MatrixObj(int l, int c) {
	sizey = l;
	sizex= c;
	sizeLin = l * c;
	//contents = new real[sizeLin];
	contents=malloc(sizeLin*sizeof(real));
}


// destructor needed to free internal structure created on memory
MatrixObj::~MatrixObj() {
	//delete[] contents;
	free(contents);
}

// Macros to simplify code
// get a value line column
#define getVal(l, c) contents[(l)*sizex + (c)]
//set value line column
#define setVal(l, c, v) contents[(l)*sizex + (c)] = (v)

// empty the MatrixObj (set with 0)
void MatrixObj::empty() {
	for (int i = 0; i < sizeLin; i++) contents[i] = 0;
}
// create identity MatrixObj
boolean MatrixObj::identity() {
	int maxi=min(sizex,sizey);
	if (maxi<1) return false;
	for (int l = 0; l < maxi; l++) 
	{
		for (int c=0; c<maxi; c++)
		{ if (l==c) contents[l * sizex+c] = 1;
			else contents[l * sizex+c] = 0;
		}
	}
	return true;
}

// compute transpose of MatrixObj A=A^T (square square part of MatrixObj)
void MatrixObj::transpose() {
	int maxi=min(sizex,sizey);
	
	for (int i = 0; i < maxi; i++)
	{
		for (int j = 0; j < i; j++) 
		{		
			real sto = getVal(i, j);
			setVal(i, j, getVal(j, i));
			setVal(j, i, sto);
		}
	}
}
// multiply MatrixObj by a real A=v*A
void MatrixObj::multiplyVal(real val) {
	for (int i = 0; i < sizeLin; i++) contents[i] *= val;
}



// M1=M2+M1
// return in self result of addition of self by M1
// only the commun part of two matrix are added
boolean addMat(real *M1,int sizey1,int sizex1,real *M2, int sizey2, int sizex2) 
{   int maxx=min(sizex1,sizex2);
	int maxy=min(sizey1,sizey2);
	if ((maxx<1)||(maxy<1))return false;
	for (int l=0; l<maxy; l++)
	{
		for (int c=0; c< maxx; c++) { M1[l*sizex1+c] += M2[l*sizex2+c]; }
	}
	//showMatMat(M1,sizey1,sizex1,&Serial);
	return true;
}

// A=A+M
// return in self result of addition of self by M
// only the commun part of two MatrixObj are added
boolean MatrixObj::add(MatrixObj M) 
{   
	return addMat(contents,sizey,sizex,M.contents,M.sizey,M.sizex);
}

// M1=M1xM2
// only the possible parts able to multiply are treated;
boolean multiplyMat(real *M1,int sizey1,int sizex1,real *M2, int sizey2, int sizex2) 
{
	int maxcol=sizex2;
	int maxline=sizey1;
	if ((maxcol<1)||(maxline<1)) return false;
	int maxop=min(sizex1,sizey2);
	if (maxop<1) return false;
	
	real result[maxline*maxcol];
	int maxprod=min(sizex1,sizey2);
	for (int l = 0; l < maxline; l++)
	{for (int c = 0; c < maxcol; c++) {
			real v=0;
			for (int k = 0; k < maxop; k++) {
				v += M1[l*sizex1+k] * M2[k*sizex2+c];
			}
			result[l*maxcol+c]=v;
		}
	}
	
	// fill inital MatrixObj with values
	for (int l = 0; l < maxline; l++) memcpy(&M1[l*sizex1],&result[l*maxcol],maxcol*sizeof(real));
	return true;
}

// A=AxM
// return in self result of multiplication of self by M
// only the possible parts able to multiply are treated;
boolean MatrixObj::multiply(MatrixObj M) {
return multiplyMat(contents,sizey,sizex,M.contents,M.sizey,M.sizex);
}


// how to get a value from MatrixObj
real MatrixObj::getValMat(int line, int column)
{if ((line<sizey) && (column<sizex))
	{return getVal(line,column);}
	return 0;
}

// to set a value to the MatrixObj
boolean MatrixObj::setValMat(int line, int column, real V)
{
if ((line<sizey) && (column<sizex))
	{
setVal(line,column,V); return true;}
	return false;
}

// to get a value of the system's solution (if exist)  
real MatrixObj::getValSol(int line)
{if ((line<sizey) && (sizex>sizey+1))
	{return getVal(line,sizex-1);}
	return 0;
};

// to set a value of constant vector (n indicate which constant if many)
boolean MatrixObj::setValConst(int n, int l,real V)
{if ((n<sizey) && (sizex>sizey+1) && (n>=0) && (n<sizex-sizey-1))
	{setVal(l,sizey+n,V); return true;}
	return false;
}
// to eliminate small values
void MatrixObj::zeroed(real v)
{
	for (int i=0; i<sizeLin; i++) { if (abs(contents[i])<v) contents[i]=0;}
}
// to copy the MatrixObj
// only commun part of MatrixObj are copied
boolean MatrixObj::copy(MatrixObj M)
{int maxx=min(sizex,M.sizex);
 int maxy=min(sizex,M.sizey);
 if ((maxx>0)&&(maxy>0))
 {
	 for (int l=0; l<maxy; l++) memcpy(&contents[l*sizex],&M.contents[l*M.sizex],maxx*sizeof(real));
	 return true;
 }
 return false;
}

// Show a MatrixObj on Serial beam
// in entry : the serial beam to be used example : showMat(Serial);
void MatrixObj::showMat(HardwareSerial *ser=&Serial) 
{
	showMatMat(contents,sizey,sizex, ser);
}

// clear vector solution (set to 0) if exist
void MatrixObj::clearSolution()
{
	// if solution exist => sizex=sizey+2 at the minimum 
	if (sizex>sizey+1)
	{for (int i=0; i<sizey; i++) contents[sizex*(i+1)-1]=0;
	}
}

// Show a MatrixObj on Serial beam
// in entry : The MatrixObj (an array), the number of lines, the number of column
// the serial beam to be used example : showMat(M,4,6,Serial);
void showMatMat(real* M, int sizey,int sizex,HardwareSerial *ser=&Serial) {
	ser->println();
	for (int i = 0; i < sizey; i++) {
		for (int j = 0; j < sizex; j++) {
			ser->print(M[i * sizex + j]);
			ser->print("\t");
		}
		ser->println();
	}
}


// show a square MatrixObj on Serial beam
void showSquareMat(real* M, int size,HardwareSerial *ser=&Serial) {
	showMatMat(M,size,size,ser);}

// show a vector on Serial beam
void showVector(real* M, int size,HardwareSerial *ser=&Serial) {
	showMatMat(M,size,1,ser);}

#undef getVal
#undef setVal
	
// *****************************************************
// *************** determinant methodes  ***************
// *****************************************************

// compute classic déterminant
// input M a rectangular MatrixObj sizey lines sizex columns
// only the square part is trated. The MatrixObj became unchanged
// if sizex=-1 (default value), the MatrixObj is taken as square (sizex=sizey)
real deterClassicMat(real* M, int sizey, int sizex=-1) 
{
	#define M(x,y) M[(x)*sizex+(y)]
	#define CO(x,y) CO[(x)*sizeC+(y)]
	if ((sizey<=0)||(M==nullptr)) return 0; // safety control
	
	if (sizex==-1) sizex=sizey; // assume square MatrixObj
	// direct results
	int maxi=min(sizex,sizey);
	if (maxi==3) 	return M[0]*(M(1,1)*M(2,2)-M(2,1)*M(1,2))
	-M[1]*(M(1,0)*M(2,2)-M(2,0)*M(1,2))
	+M[2]*(M(1,0)*M(2,1)-M(2,0)*M(1,1));
	if (maxi == 2) return  M[0] * M(1,1) - M[1] * M(1,0); 
	if (maxi == 1) return M[0];
	
	// at this point size is 4 minimum
	real det = 0;
	// developping by first line
	for (int col = 0; col < maxi; col++) {
		if (M[col]!=0)
		{// creating submatrix
			int sizeC = maxi - 1;
			real CO[sizeC*sizeC];
			
			// filling submatrix C colum by column
			// avoiding col column and 0 line 
			int nc = 0;
			for (int c = 0; c < maxi; c++) {
				if (c != col) {
					for (int l = 1; l < maxi; l++) CO(l-1,nc) = M(l,c);
					nc++;
				}
			}
			
			/*
			// filling subMatrix line by lines 
			// avoiding column col 
			int max=maxi-1;
			for (int l=0; l<max; l++)
			{
				if (col>0) memcpy(&CO(l,0),&M(l+1,0),col*sizeof(real));
				if (col<max) memcpy(&CO(l,col),&M(l+1,col+1),(max-col)*sizeof(real));
			}
			*/
			if (col % 2 == 0) det += M[col] * deterClassicMat(CO, sizeC);
			else det -= M[col] * deterClassicMat(CO, sizeC);
		}
	}
	return det;
	#undef M
	#undef CO
}

// inverted the MatrixObj  M^-1=1/det(M)*C^T where C is the comatrix of M
// ci,j=(-1)^(i+j)det(CO) where CO is the MatrixObj formed by terms of inital MatrixObj exept line i and column j
// CO is a MatrixObj (size-1 x size-1)
// input M a rectangular MatrixObj sizey lines sizex columns
// only the square part is inverted. The rest of MatrixObj became unchanged
// if sizex=-1 (default value), the MatrixObj is taken as square (sizex=sizey)
boolean invMatrixClassicMat(real* M, int sizey, int sizex) 
{
	#define M(x,y) M[(x)*sizex+(y)]
	#define invMat(x,y) invMat[(x)*maxi+(y)]
	#define CO(x,y) CO[(x)*sizeC+(y)]
	if (M==nullptr) return false; // safety control
	int maxi=min(sizex,sizey);
	if (maxi<0) return false;
	if ((maxi==1)&&(abs(M[0])>_Epsilon)) {M[0]=1/M[0]; return true;}
	real det = deterClassicMat(M,sizey,sizey); // determinant of initial MatrixObj
	if (abs(det)<_Epsilon) return false;  // singular MatrixObj could not be inverted
	real invMat [maxi*maxi]; // inverted MatrixObj
	if (maxi==2) 
	{
		invMat(0,0)=M(1,1)/det;
		invMat(1,1)=M(0,0)/det;
		invMat(0,1)=-M(0,1)/det;
		invMat(1,0)=-M(1,0)/det;
		M(0,0)=invMat(0,0);
		M(0,1)=invMat(0,1);
		M(1,0)=invMat(1,0);
		M(1,1)=invMat(1,1);
		return true;
	}
	// at this point maxi>2
	for (int l = 0; l < maxi; l++) 
	{
		for (int c=0; c<maxi; c++)
		{// create and fill sub MatrixObj 
			int sizeC=maxi-1;
			real CO[sizeC*sizeC]; // creating comatrix
			for (int nl=0;nl<sizeC+1; nl++)
			{for (int nc=0; nc<sizeC+1; nc++)
				{if (nl<l)
					{if (nc<c) { CO(nl,nc)=M(nl,nc);}
						else if (nc>c) { CO(nl,nc-1)=M(nl,nc);}
						
					}
					else if (nl>l)
					{if (nc<c) { CO(nl-1,nc)=M(nl,nc);}
						else if (nc>c) { CO(nl-1,nc-1)=M(nl,nc);}			
					}			
				}
			}
						
			if ((c+l)%2==0)invMat(c,l)=deterClassicMat(CO,sizeC,sizeC) / det;  
			else invMat(c,l)=-deterClassicMat(CO,sizeC,sizeC) / det;// fill and transpose				
		}
	}
	// back invMat to M
	for (int l = 0; l < maxi; l++) {memcpy(&M[l*sizex],&invMat[l*maxi],maxi*sizeof(real));}
	
	return true;       // succed !!
	#undef M
	#undef invM
	#undef CO
}


// solve the system MX=B with the kramer méthode
// return determinant of M (if 0 the solution does not exist)
// due to large amount of memory used and
// huge number of opérations, this function is
// only for purist and realy NOT to USE in real problems
// M is a rectangular MatrixObj 
// M contains M,Bi, X M is a rectangular MatrixObj
// constant column indicate the constant column to take constant column from 0 to ...
// last column will store the solution vector
// asumes sizex>sizey+2 (in order to store at least 1 constant column and a vector solution)
real deterSolveMat(real* M, int sizey, int sizex, int constantColumn) {
	#define M(i, j) M[(i)*sizex + j]
	#define B(i) M((i),sizey+constantColumn)
	#define X(i) M((i),sizex-1)
	// safety controls
	if ((sizey<=0)||(M==nullptr)) return 0; 
	if (sizex<sizey+2+constantColumn) return 0;

// let us go !
	real deterM = deterClassicMat(M, sizey, sizex);  // The main determinant
	if (abs(deterM) < _Epsilon) return 0;
	// compute for each solution's member
	for (int i = 0; i < sizey; i++) {
		// swap columns between MatrixObj and constants
		for (int l = 0; l < sizey; l++) {
			real sto = B(l);
			B(l) = M(l, i);
			M(l, i) = sto;
		}
		// compute solution with Kramer méthode
		X(i) = deterClassicMat(M, sizey, sizex) / deterM;
		// back swap columns
		for (int l = 0; l < sizey; l++) {
			real sto = B(l);
			B(l) = M(l, i);
			M(l, i) = sto;
		}
	}
	return deterM;	
#undef X
#undef B
#undef M
	
}

// object treatment (encapsulation!)
// derterminant treatment
real MatrixObj::deterClassic()
{
	return deterClassicMat(getContents(),sizey,sizex);
}

boolean MatrixObj::invMatrixClassic()
{
	return invMatrixClassicMat(getContents(),sizey,sizex);
}

real MatrixObj::deterSolve(int nconst=0)
{
	return deterSolveMat(getContents(),sizey,sizex,nconst);
}

// *********************************************************
// *************** Gauss Jordan osition **************
// *********************************************************

// compute determinant by gauss jordan elimination
// return 0 if problem
real gaussJordanDeterMat(real* M, int sizey, int sizex)
{
	return gaussJordanPivMat(M, sizey, sizex,-1);
}

// Gauss Jordan transformation with partial pivoting
// System M X = Bi all store in one MatrixObj
// M is a rectangular MatrixObj
// M contains M,Bi, X M is a rectangular MatrixObj
// constant column indicate the constant column to take constant column from 0 to ...
// asumes sizex>sizey+2 (in order to store at least 1 constant column and a vector solution)
real gaussJordanPivMat(real* M, int sizey, int sizex,int constantColumn) {
#define M(i, j) M[(i)*sizex + (j)]
	if (sizey<1) return 0;
	if (sizex<1) return 0;
	int maxcol=0;
	// safety control
	if (constantColumn<0) maxcol=sizex; else maxcol=sizex-1;
    int maxline=min(sizey,maxcol);
		// let's go on !
	// simple cases
	if (maxline == 1) {
		if (abs(M[0]) > _Epsilon) {
			return M[0];  //M(0,0);
		} else return 0;
	}
	// general case
	real coefDeter = 1;
	for (int l = 0; l < maxline; l++) {
		// search max pivot under the diagonal value
		int piv = l;
		if (l < maxline - 1) {
			real maxi = abs(M(l, l));
			for (int nl = l + 1; nl < maxline; nl++) {
				real newMaxi = abs(M(nl, l));
				if (newMaxi > maxi) {
					piv = nl;
					maxi = newMaxi;
				}
			}
			// pivot line is piv index line
			if (maxi < (_Epsilon)) return 0;  // singular MatrixObj impossible to treat determinant =0;
			// inversion of lines and normalize to 1 line l

			if (l != piv) {  // inversion
				for (int c = l; c < maxcol; c++) {
					real sto = M(piv, c);
					M(piv, c) = M(l, c);
					M(l, c) = sto;
				}
				coefDeter *= -1;  // determinant changes sign if we toggle any lines or any column
			}
		}
		// normalise line
		real pivot = M(l, l);
		coefDeter *= pivot;
		M(l, l) = 1;
		for (int c = l + 1; c < maxcol; c++) {
			M(l, c) /= pivot;
		}
		if (l < maxline - 1) {
			// modification of lines under diagonal line
			for (int nl = l + 1; nl < maxline; nl++) {
				real C = M(nl, l) / M(l, l);  // coef for multipying should be (M(nl, l)/M(l,l))
				// but M(l,l) has been normalized to 1

				// scanning all the line
				for (int nc = l + 1; nc < maxcol; nc++) M(nl, nc) -= M(l, nc) * C;
				// treating constant
				M(nl, l) = 0;  // whithout computing it is 0
			}
		}
	}
	return coefDeter;  // it is the determinant of initial MatrixObj !!
#undef M
}

// solve system  by backward computing asuming M is an upper triangle MatrixObj
// added with constant Bi folowed by a X
void gaussJordanFinalSolveMat(real* M, int sizey, int sizex, int constantColumn) {
	// resolution by backward computing
	// begining with the last line which contain a simple équation M[l][l]*X[l]=B[l]
	// which leads to X[l]=B[I]/M[I][I] assumes that M[I][I] is not null
	// this case could not occures at this point of computing.
	// in fact it's value is 1 due to previous manipulation
	// for the others lines, we compute the somprod of known values of previous lines computed and
	// X[l]=(B[l]-someprod)/ M[l][l]
	// as M[l][l] is normalized to 1 by previous manipulations, it is not used to make division

	#define M(i, j) M[(i)*sizex + j]
	#define B(i) M((i),sizey+constantColumn)
	#define X(i) M((i),sizex-1)
	X(sizey - 1) = B(sizey - 1);  // downest values of vector solution
	// others values
	for (int l = sizey-2; l >= 0; l--) {
		real sumprod = 0;
		for (int c = sizey - 1; c > l; c--) sumprod += M(l, c) * X(c);
		X(l) = B(l) - sumprod;  // asume that diagonal term of matris is 1
	}
#undef X
#undef B
#undef M
}

// solves the systemn MX=Bi M a rectangular MatrixObj wich contain M, Bi, X
// return determinant of square part of M (if 0 the solution does not exist)
real gaussJordanSolveMat(real* M, int sizey, int sizex, int constantColumn) {
	real det = gaussJordanPivMat(M, sizey,sizex,constantColumn);
	if (abs(det) < _Epsilon) return 0;
	gaussJordanFinalSolveMat(M, sizey, sizex, constantColumn);
	return det;
}

// object treatment (encapsulation!)
// gauss jordan treatment
real MatrixObj::gaussJordanDeter()
{
	return gaussJordanPivMat(contents, sizey,  sizex,-1);
	}

real MatrixObj::gaussJordanPiv(int constColumn)
{
	return gaussJordanPivMat(contents, sizey, sizex,constColumn);
	}

void MatrixObj::gaussJordanFinalSolve(int constColumn)
{
	return gaussJordanFinalSolveMat(contents,sizey,sizex,constColumn);
}

real MatrixObj::gaussJordanSolve(int constColumn)
{
	return gaussJordanSolveMat(contents,sizey,sizex,constColumn);
}

// *****************************************************
// *************** LU Crout decomposition **************
// *****************************************************

// compute determinant by LU Crout pivoting elimination
// return 0 if problem
real luCroutDeterMat(real* M, int sizey, int sizex)
{
	return luCroutPivMat(M, sizey, sizex,-1);
}

// make a LU decomposition with no pivoting
// L : lower triangular MatrixObj
// U : upper triangular MatrixObj with 1 on diagonal
// LU=M
// return determinant of M
// pay attention that osition could crash
// due to momentary 0 on diagonal
real luCroutMat(real* M, int sizey, int sizex=-1) {
#define M(i, j) M[(i)*sizex + (j)]
    if (sizex<0) sizex=sizey; // assume a square MatrixObj
	if (sizey<sizex) return 0; // error
	real s;
	real tmp;
	for (int k = 0; k < sizey; k++) {
		for (int i = k; i < sizey; i++) {
			s = 0.0;
			for (int j = 0; j < k; j++)
			s += M(i, j) * M(j, k);
			M(i, k) -= s;
		}
		if (abs(M(k, k)) < _Epsilon) { return 0; }
		tmp = 1.0 / M(k, k);
		for (int j = k + 1; j < sizey; j++) {
			s = 0.0;
			for (int i = 0; i < k; i++)
			s += M(k, i) * M(i, j);
			M(k, j) = (M(k, j) - s) * tmp;
		}
	}
	real deter = 1;
	for (int i = 0; i < sizey; i++) deter *= M(i, i);
	return deter;
#undef M
}

// make a LU decomposition with  partial pivoting
// LU Crout transformation with partial pivoting
// System M X = Bi all store in one MatrixObj
// M is a rectangular MatrixObj
// M contains M,Bi, X M is a rectangular MatrixObj
// constant column indicate the constant column to take constant column from 0 to ...
// asumes sizex>sizey+2 (in order to store at least 1 constant column and a vector solution)
// L : lower triangular MatrixObj
// U : upper triangular MatrixObj with 1 on diagonal
// LU=M
// B : constant second member of the equation M X = B
// return determinant of M
// pay attention that after transformation deter LU may differ from deter Minit by sign due to 
// line swapping
// return determinant of initial MatrixObj
real luCroutPivMat(real* M, int sizey,int sizex,int constantColumn=-1)
{
#define M(i, j) M[(i)*sizex + j]
    if (sizey<1) return 0;
	if (sizex<1) return 0;
	int maxcol=0;
	// safety control
	if (constantColumn<0) maxcol=sizey; else maxcol=sizex-1;

	if (maxcol>sizex) return 0;

	// let's go on !
	// simple cases
	if (sizey == 1) {
		if (abs(M[0]) > _Epsilon) {
			return M[0];  //M(0,0);
		} else return 0;
	}
	real s;
	real tmp;
	int deterSign = 1;
	
	for (int k = 0; k < sizey; k++) {
		// searching best pivot
		int piv = k;
		real maxi = abs(M(k, k));
		for (int nl = k + 1; nl < sizey; nl++) {
			real v = abs(M(nl, k));
			if (v > maxi) {
				maxi = v;
				piv = nl;
			}
		}
		// swap lines if necessary MatrixObj coef and constant 
		if (piv != k) {
			real sto = 0;
			// pay attention in gauss methode you have to swap line from column k to size-1
			// because terms under diagonal have been set to 0
			// here you have to swap all line column from 0 to size-1
			for (int nc = 0; nc < maxcol; nc++) {
				sto = M(piv, nc);
				M(piv, nc) = M(k, nc);
				M(k, nc) = sto;
			}
			deterSign *= -1;
		}
		// go on compute the osition
		for (int i = k; i < sizey; i++) {
			s = 0.0;
			for (int j = 0; j < k; j++)
			s += M(i, j) * M(j, k);
			M(i, k) -= s;
		}
		if (abs(M(k, k)) < _Epsilon) { return 0; }
		tmp = 1.0 / M(k, k);
		for (int j = k + 1; j < sizey; j++) {
			s = 0.0;
			for (int i = 0; i < k; i++)
			s += M(k, i) * M(i, j);
			M(k, j) = (M(k, j) - s) * tmp;
		}
	}
	real deter = 1;
	for (int i = 0; i < sizey; i++) deter *= M(i, i);
	return deter * deterSign; // taking care of swapping
#undef M
}
// solves the system M X = Bi
// assuming M is in LU Crout mode
// L is lower triangle
// U is upper triangle with 1 on diagonal
// it gives solution of multiples constant vectors stored with the MatrixObj and
// treated in the same time as MatrixObj itself
// constantColumn varies from 0 to (sizex-sizey-2)

void luCroutFinalSolveMat(real * M, int sizey, int sizex, int constantColumn)
{
	// resolution by double down and up computing
#define L(x, y) M[(x)*sizex + (y)]
#define U(x, y) M[(x)*sizex + (y)]
#define B(x) M[(x)*sizex+sizey+constantColumn]
#define X(x) M[((x)+1)*sizex-1]
	// solve L Y = B
	real Y[sizey];
	Y[0] = B(0)/L(0,0);  // uppest
	// others values
	for (int l = 1; l < sizey; l++) {
		real sumprod = 0;
		for (int c = 0; c < l; c++) sumprod += L(l, c) * Y[c];
		Y[l] = (B(l) - sumprod)/L(l,l);  
	}
	// solve U X = Y
	// this time the solution mechanism is backward assuming U(l,l)=1
	X(sizey - 1) = Y[sizey - 1] ;  // lowest
	// others values
	for (int l = sizey - 2; l >= 0; l--) {
		real sumprod = 0;
		for (int c = l +1; c < sizey; c++) sumprod += U(l, c) * X(c);
		X(l) = Y[l]-sumprod;  // asume that diagonal term of MatrixObj is 1
	}
#undef X
#undef B
#undef U
#undef L
	
}
// solves the systemn MX=Bi M a rectangular MatrixObj wich contain M, Bi, X
// return determinant of square part of M (if 0 the solution does not exist)
real luCroutSolveMat(real* M, int sizey, int sizex, int constantColumn) {
	real det = luCroutPivMat(M, sizey,sizex,constantColumn);
	if (abs(det) < _Epsilon) return 0;
	luCroutFinalSolveMat(M, sizey, sizex, constantColumn);
	return det;
}

// object treatment (encapsulation!)
// LU treatment
real MatrixObj::luCroutDeter()
{
	return luCroutPivMat(contents, sizey, sizex,-1);
	}

real MatrixObj::luCroutPiv(int constColumn)
{
	return luCroutPivMat(contents, sizey, sizex,constColumn);
	}

void MatrixObj::luCroutFinalSolve(int constColumn)
{
	return luCroutFinalSolveMat(contents,sizey,sizex,constColumn);
}

real MatrixObj::luCroutSolve(int constColumn)
{
	return luCroutSolveMat(contents,sizey,sizex,constColumn);
}